package cz.mendelu.dnaAnalyser.analyse.palindrome;

import cz.mendelu.dnaAnalyser.analyse.palindrome.model.PalindromeModel;
import cz.mendelu.dnaAnalyser.analyse.palindrome.model.StabilityModel;
import cz.mendelu.dnaAnalyser.sequence.SequenceService;
import cz.mendelu.dnaAnalyser.utils.controller.DeleteMapping;
import cz.mendelu.dnaAnalyser.utils.controller.GetMapping;
import cz.mendelu.dnaAnalyser.utils.controller.*;
import cz.mendelu.dnaAnalyser.utils.controller.PostMapping;
import cz.mendelu.dnaAnalyser.utils.controller.PutMapping;
import cz.mendelu.dnaAnalyser.utils.swagger.ApiPaginationParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;

@RestController
@Api(tags = "api/analyse/palindrome")
@RequestMapping("api/analyse/palindrome")
public class PalindromeAnalyseController {

    @Data
    static class PalindromeAnalyseRequest {

        private List<UUID> sequences;

        private Boolean dinucleotide;

        private String size = "6-30";

        private String spacer = "0-10";

        private String mismatches = "0,1";

        private Set<String> tags;

        private StabilityModel stabilityModel;

    }


    @Autowired
    private PalindromeAnalyseService palindromeAnalyseService;

    @Autowired
    private SequenceService sequenceService;

    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAutheticated()")
    @ApiPaginationParams
    @ApiOperation("Get page with PalindromeModel analysis")
    public ResponsePage<PalindromeModel> getPageWithPalindromeAnalysis(
            PaginationParam paginationParam,
            SortParam sortParam,
            TagFilterParam tagFilterParam
    ){

        throw new UnsupportedOperationException("Not implemented yet.");
    }

    @GetMapping("/tag")
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiPaginationParams
    @ApiOperation("Get all tags defined for sequences")
    public ResponsePage<String> getPageWithTags(
            PaginationParam paginationParam,
            SortParam sortParam
    ) {
        throw new UnsupportedOperationException("Not implemented yet.");
    }

    @GetMapping("/{id}")
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Get one PalindromeModel analyse by ID.")
    public Response<PalindromeModel> getOnePalindromeAnalyse(
            @PathVariable UUID id)
    {
        throw new UnsupportedOperationException("Not implemented yet.");
    }

    @PostMapping()
    @ResponseStatus(HttpStatus.CREATED)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Create palindrome analyze.")
    public ResponseList<PalindromeAnalyse> postPalindromeAnalyse(
            @ApiIgnore Authentication authentication,
            @RequestBody @Valid PalindromeAnalyseRequest request
    ) {

        List<PalindromeAnalyse> analysis = request.getSequences()
                .stream()
                .map(uuid ->
                        palindromeAnalyseService.executePalindromeAnalyse(null /*
                                PalindromeModel.builder()
                                        .sequence(sequenceService.findOne(uuid))
                                        .owner(AuthenticationUtils.toUser(authentication))
                                        .tags(request.getTags())
                                        .mismatches(request.getMismatches())
                                        .size(request.getSize())
                                        .spacer(request.getSpacer())
                                        .stabilityModel(request.getStabilityModel())
                                        .build() */
                        )
                ).collect(Collectors.toList());

        return ResponseList.responseList(analysis);
    }

    @PutMapping("/{id}/tags")
    @ResponseStatus(HttpStatus.ACCEPTED)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Modify tags")
    public Response<PalindromeModel> modifyTags(
            @PathVariable UUID id,
            @RequestBody TagModifyRequest request)
    {
        throw new UnsupportedOperationException("Not implemented yet.");
    }

    @DeleteMapping("/{id}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Delete one PalindromeModel analyse by ID")
    public void deletePalindrome(
            @PathVariable UUID id)
    {
        throw new UnsupportedOperationException("Not implemented yet.");
    }

}
