package cz.mendelu.dnaAnalyser.sequence.annotation.feature;


import cz.mendelu.dnaAnalyser.utils.analyse.data.DataLocalisedMapper;
import cz.mendelu.dnaAnalyser.utils.analyse.data.DataMapper;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.cursor.Cursor;
import org.apache.ibatis.session.RowBounds;

import java.util.List;

public interface FeatureMapper extends DataMapper<Feature> {

    // language=SQL
    String TABLE = "CREATE TABLE FEATURE (" +
            "ID integer NOT NULL AUTO_INCREMENT, " +
            "FEATURETYPE varchar, " +
            "STARTPOSITION varchar, " +
            "ENDPOSITION varchar, " +
            "ANNOTATION varchar, " +
            "PRIMARY KEY (ID))";

    // language=SQL
    @Select("SELECT count(*) FROM FEATURE")
    int countAll();

    // language=SQL
    @Select("SELECT * FROM FEATURE")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "featureType", column = "FEATURETYPE"),
            @Result(property = "startPosition", column = "STARTPOSITION"),
            @Result(property = "endPosition", column = "ENDPOSITION"),
            @Result(property = "annotation", column = "ANNOTATION"),

    })
    List<Feature> getAll(RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM FEATURE")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "featureType", column = "FEATURETYPE"),
            @Result(property = "startPosition", column = "STARTPOSITION"),
            @Result(property = "endPosition", column = "ENDPOSITION"),
            @Result(property = "annotation", column = "ANNOTATION"),
    })
    Cursor<Feature> getCursor();


    // language=SQL
    @Select("SELECT * FROM FEATURE WHERE ID = #{id}")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "featureType", column = "FEATURETYPE"),
            @Result(property = "startPosition", column = "STARTPOSITION"),
            @Result(property = "endPosition", column = "ENDPOSITION"),
            @Result(property = "annotation", column = "ANNOTATION"),
    })
    Feature getById(int id, RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM FEATURE WHERE ANNOTATION = #{annotation}")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "featureType", column = "FEATURETYPE"),
            @Result(property = "startPosition", column = "STARTPOSITION"),
            @Result(property = "endPosition", column = "ENDPOSITION"),
            @Result(property = "annotation", column = "ANNOTATION"),
    })
    List<Feature> getByAnnotation(String annotation);

    // language=SQL
    @Update("UPDATE FEATURE SET STARTPOSITION = #{startPosition}, ENDPOSITION = #{endPosition}, FEATURETYPE = #{featureType}, SCORE = #{score}, ANNOTATION = #{annotation} WHERE ID = #{id}")
    void update(Feature feature);

    // language=SQL
    @Delete("DELETE from FEATURE WHERE ID = #{id}")
    void delete(int id);

    // language=SQL
    @Insert("INSERT INTO FEATURE (ID, FEATURETYPE, STARTPOSITION, ENDPOSITION, ANNOTATION) VALUES (#{id}, #{featureType}, #{startPosition}, #{endPosition}, #{annotation})")
    @Options(useGeneratedKeys = true, keyProperty = "id")
    void insert(Feature feature);

}

