package db.migration;

import org.flywaydb.core.api.migration.BaseJavaMigration;
import org.flywaydb.core.api.migration.Context;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.util.UUID;
import java.util.List;
import java.util.ArrayList;


public class V12__AlterUsers extends BaseJavaMigration {

    private static final Logger LOG = LoggerFactory.getLogger(V12__AlterUsers.class);

    public void migrate(Context context) throws Exception {

        List<String> duplicates = new ArrayList<>();
        List<UUID> idsToDelete = new ArrayList<>();


        try (Statement select = context.getConnection().createStatement()) {
            /*
             * SQL #1: Select duplicate usernames 
             */
            try (ResultSet rs = select.executeQuery(
                "SELECT username FROM users  " +
                "WHERE username NOT IN ('admin@mendelu.cz', 'user@mendelu.cz', 'disabled@mendelu.cz') " + // default accounts imported through migrations with NULL created_date
                "GROUP BY username " +
                "HAVING COUNT(username) > 1;"
                )) {
                while (rs.next()) {
                    duplicates.add(rs.getString(1));
                    LOG.info("Added duplicate entry for {}.", rs.getString(1));
                }
            }
        }
        for (String dup : duplicates) {

            /*
            * SQL #2: Select all IDS for every duplicate
            */
            String sql = "SELECT id, username, created_date FROM users  " +
                         "WHERE username = ? " +
                         "ORDER BY created_date ASC;";

            try (PreparedStatement selectIds = context.getConnection().prepareStatement(sql)) {
                selectIds.setString(1, dup);
                try (ResultSet rsSelect = selectIds.executeQuery()) {

                    // !IMPORTANT
                    rsSelect.next(); // <- skips the first row in result (which is the user we want to keep)
                    LOG.info("Skipping latest duplicate - valid user ({} - {} [{}]).", rsSelect.getString(2), rsSelect.getString(1), rsSelect.getString(3));

                    LOG.info("Listing duplicate accounts to DELETE: ");
                    while (rsSelect.next()) {
                        UUID uid = rsSelect.getObject(1, java.util.UUID.class);
                        LOG.info("\tuuid: {}, username: {}, created_date: {}", uid.toString(), rsSelect.getString(2), rsSelect.getString(3));
                    }
                }
            }
        }
    }
}

